@@ -15,6 +15,13 @@ use tracing::debug;
 pub struct FieldTable {
     keys: KeyMap<Value, BumpProxy>,
     values: KeyMap<Value, BumpProxy>,
+    alloc: BumpProxy,
+}
+
+impl Clone for FieldTable {
+    fn clone(&self) -> Self {
+        Self::from_iter(self.iter(), self.alloc.clone()).unwrap()
+    }
 }
 
 impl std::fmt::Debug for FieldTable {
@@ -37,7 +44,8 @@ impl FieldTable {
     pub fn with_capacity(size: usize, proxy: BumpProxy) -> Result<Self, MapError> {
         let res = Self {
             keys: KeyMap::with_capacity(size, proxy.clone())?,
-            values: KeyMap::with_capacity(size, proxy)?,
+            values: KeyMap::with_capacity(size, proxy.clone())?,
+            alloc: proxy,
         };
         Ok(res)
     }
@@ -62,6 +70,20 @@ impl FieldTable {
         self.values.get_mut(handle)
     }
 
+    pub fn from_iter(
+        it: impl Iterator<Item = (Value, Value)>,
+        alloc: BumpProxy,
+    ) -> Result<Self, ExecutionErrorPayload> {
+        let mut result = Self::with_capacity(it.size_hint().0, alloc)
+            .map_err(|_err| ExecutionErrorPayload::OutOfMemory)?;
+
+        for (key, value) in it {
+            result.insert(key, value)?;
+        }
+
+        Ok(result)
+    }
+
     pub fn insert(&mut self, key: Value, value: Value) -> Result<(), ExecutionErrorPayload> {
         let handle = Self::hash_value(key)?;
         self.keys
@@ -86,7 +108,7 @@ impl FieldTable {
                 Handle::from_str(s).unwrap()
             }
             Value::Integer(i) => Handle::from(i),
-            Value::Floating(_) | Value::Object(_) => return Err(ExecutionErrorPayload::Unhashable),
+            Value::Real(_) | Value::Object(_) => return Err(ExecutionErrorPayload::Unhashable),
         };
         Ok(handle)
     }
@@ -100,13 +122,30 @@ impl FieldTable {
                 (*k, *v)
             })
     }
+
+    pub fn iter_mut(&mut self) -> impl Iterator<Item = (Value, &mut Value)> + '_ {
+        self.keys
+            .iter()
+            .zip(self.values.iter_mut())
+            .map(|((k1, k), (k2, v))| {
+                debug_assert!(k1 == k2);
+                (*k, v)
+            })
+    }
 }
 
 pub struct RuntimeData {
-    pub(crate) stack: ValueStack,
+    pub(crate) value_stack: ValueStack,
     pub(crate) call_stack: BoundedStack<CallFrame>,
     pub(crate) global_vars: Vec<Value>,
     pub(crate) memory: BumpProxy,
+    pub(crate) object_list: Vec<NonNull<FieldTable>>,
+}
+
+impl Drop for RuntimeData {
+    fn drop(&mut self) {
+        self.clear();
+    }
 }
 
 pub(crate) struct CallFrame {
@@ -124,9 +163,10 @@ impl RuntimeData {
     ) -> Result<Self, ExecutionErrorPayload> {
         let memory: BumpProxy = BumpAllocator::new(memory_capacity).into();
         let res = Self {
-            stack: ValueStack::new(stack_size),
+            value_stack: ValueStack::new(stack_size),
             call_stack: BoundedStack::new(call_stack_size),
             global_vars: Vec::with_capacity(16),
+            object_list: Vec::with_capacity(16),
             memory,
         };
         Ok(res)
@@ -135,11 +175,6 @@ impl RuntimeData {
     /// Initialize a new cao-lang table and return a pointer to it
     pub fn init_table(&mut self) -> Result<NonNull<FieldTable>, ExecutionErrorPayload> {
         unsafe {
-            let alloc = self.memory.clone();
-            let table = FieldTable::with_capacity(16, alloc).map_err(|err| {
-                debug!("Failed to init table {:?}", err);
-                ExecutionErrorPayload::OutOfMemory
-            })?;
             let table_ptr = self
                 .memory
                 .alloc(Layout::new::<FieldTable>())
@@ -147,15 +182,28 @@ impl RuntimeData {
                     debug!("Failed to allocate table {:?}", err);
                     ExecutionErrorPayload::OutOfMemory
                 })?;
+            let table = FieldTable::with_capacity(8, self.memory.clone()).map_err(|err| {
+                debug!("Failed to init table {:?}", err);
+                ExecutionErrorPayload::OutOfMemory
+            })?;
 
-            std::ptr::write(table_ptr.as_ptr() as *mut FieldTable, table);
+            let table_ptr: NonNull<FieldTable> = table_ptr.cast();
+            std::ptr::write(table_ptr.as_ptr(), table);
+            self.object_list.push(table_ptr);
 
-            Ok(std::mem::transmute(table_ptr))
+            Ok(table_ptr)
         }
     }
 
     pub fn clear(&mut self) {
-        self.stack.clear();
+        for obj in self.object_list.iter_mut() {
+            unsafe {
+                std::ptr::drop_in_place(obj.as_ptr());
+            }
+        }
+
+        self.object_list.clear();
+        self.value_stack.clear();
         self.global_vars.clear();
         self.call_stack.clear();
         unsafe {
@@ -172,7 +220,10 @@ impl RuntimeData {
     }
 
     /// Types implementing Drop are not supported, thus the `Copy` bound
-    pub fn write_to_memory<T: Sized + Copy>(&mut self, val: T) -> Result<*mut T, ExecutionErrorPayload> {
+    pub fn write_to_memory<T: Sized + Copy>(
+        &mut self,
+        val: T,
+    ) -> Result<*mut T, ExecutionErrorPayload> {
         let l = std::alloc::Layout::new::<T>();
         unsafe {
             let ptr = self
