@@ -93,7 +93,7 @@ def test_bad_json_is_value_error():
 def test_recursion_limit():
     program = {"submodules": {}, "lanes": {"main": {"cards": []}}}
     _pr = program
-    for i in range(2):
+    for _ in range(2):
         _pr["submodules"]["foo"] = {"submodules": {}, "lanes": {}}
         _pr = _pr["submodules"]["foo"]
 
